#include <windows.h>
#include <msp/core/mutex.h>
#include <msp/core/systemerror.h>
#include "dir.h"

using namespace std;

namespace Msp {
namespace FS {

void mkdir(const Path &path, int)
{
	if(!CreateDirectory(path.str().c_str(), NULL))
		throw system_error("CreateDirectory");
}

void rmdir(const Path &path)
{
	if(!RemoveDirectory(path.str().c_str()))
		throw system_error("RemoveDirectory");
}

/* Windows documentation says Get/SetCurrentDirectory use a global buffer and
are not thread safe. */
static Mutex &cwd_mutex()
{
	static Mutex mutex;
	return mutex;
}

Path getcwd()
{
	MutexLock lock(cwd_mutex());

	char buf[1024];
	Path result;
	DWORD len = GetCurrentDirectory(sizeof(buf), buf);
	if(len>=sizeof(buf))
	{
		vector<char> buf2(len+1);
		len = GetCurrentDirectory(buf2.size(), buf2.data());
		result = buf2.data();
	}
	else
		result = buf;

	if(!len)
		throw system_error("GetCurrentDirectory");

	return result;
}

void chdir(const Path &path)
{
	MutexLock lock(cwd_mutex());
	if(!SetCurrentDirectory(path.c_str()))
		throw system_error("SetCurrentDirectory");
}

} // namespace FS
} // namespace Msp
